/*
===============================================================================

	FILE:	arithmeticdecoder.cpp
	
	CONTENTS:
			
    see corresponding header file

  PROGRAMMERS:
	
		martin isenburg@cs.unc.edu
	
	COPYRIGHT:
	
		(c) 2001 martin isenburg@cs.unc.edu
		
		This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	
	CHANGE HISTORY:
	
    see corresponding header file
	
===============================================================================
*/
#include "ArithmeticDecoder.h"

#include "arithmeticmodel.h"

#include <iostream.h>

ArithmeticDecoder::ArithmeticDecoder(int* bits, int number_bits)
{
	this->bits = bits;
	this->number_bits = number_bits;
	current_bit = 0;
	garbage_bits = 0;
	value = 0;
	for (int i=1; i <= 16; i++) // Code_value_bits = 16
	{
		value = 2*value + input_bit();
  }
	low = 0;
	high = 65535; // Top_value = 65535
}

ArithmeticDecoder::~ArithmeticDecoder()
{
	delete [] bits;
}

int ArithmeticDecoder::decode(ArithmeticModel* am)
{
	__int64 low_to_high;
	int cum;
	int sym;

	low_to_high = (__int64)(high-low)+1;
	cum = (int)((((__int64)(value-low)+1)*am->cfreq[0]-1)/low_to_high);

	for (sym = 0; am->cfreq[sym+1]>cum; sym++)
	{
    	/* do nothing */
	}
	
	if (sym<0 || sym>=am->nsym)
	{
		cerr << "decode: symbol out of range" << endl;
	}

	high = low + (low_to_high*am->cfreq[sym])/am->cfreq[0]-1;
	low = low +  (low_to_high*am->cfreq[sym+1])/am->cfreq[0];

	decode();
	
	am->update(sym);

	return sym;
}
			
int ArithmeticDecoder::decode(ArithmeticModel* am, int low_sym)
{
	__int64 low_to_high;
	int cum;
	int sym;

	if (low_sym<0 || low_sym>=am->nsym)
	{
		cerr << "decode: lowest symbol out of range. low_sym: " << low_sym << endl;
	}

	low_to_high = (__int64)(high-low)+1;
	cum = (int)((((__int64)(value-low)+1)*am->cfreq[low_sym]-1)/low_to_high);

	for (sym = 0; am->cfreq[sym+1]>cum; sym++)
	{
    	/* do nothing */
	}
	
	if (sym<low_sym || sym>=am->nsym)
	{
		cerr << "decode: symbol out of range. sym: " << sym << " low_sym: " << low_sym << endl;
	}

	high = low + (low_to_high*am->cfreq[sym])/am->cfreq[low_sym]-1;
	low = low +  (low_to_high*am->cfreq[sym+1])/am->cfreq[low_sym];

	decode();
	
	am->update(sym);

	return sym;
}

int ArithmeticDecoder::decode(int range)
{
	__int64 low_to_high;
	int cum;
	int sym;

	low_to_high = (__int64)(high-low)+1;
	cum = (int)((((__int64)(value-low)+1)*range-1)/low_to_high);

	for (sym = 0; (range-sym-1)>cum; sym++)
	{
    	/* do nothing */
	}
	
	if (sym<0 || sym>=range)
	{
		cerr << "decode: symbol out of range" << endl;
	}

	high = low + (low_to_high*(range-sym))/range-1;
	low = low +  (low_to_high*(range-sym-1))/range;

	decode();
	
	return sym;
}

bool ArithmeticDecoder::hasBits()
{
	return current_bit < number_bits;
}

void ArithmeticDecoder::decode()
{
	for (;;)
	{
		if (high<32768) // Half = 32768
		{
			/* do nothing */
		}
		else if (low>=32768)
		{
			value -= 32768;
			low -= 32768;
			high -= 32768;
		}
		else if (low>=16384 && high<49152) // First_qtr = 16384  Third_qtr = 49152
		{
			value -= 16384;
			low -= 16384;
			high -= 16384;
		}
		else
		{
			break;
		}
		low = 2*low;
		high = 2*high+1;
		value = 2*value + input_bit();
	}
}
	
int ArithmeticDecoder::input_bit()
{		
	if (current_bit == number_bits)
	{
		garbage_bits += 1;
    if (garbage_bits > 16-2) // Code_value_bits = 16
		{
			cerr << "bad input file" << endl;
		}
		return 0; // garbage bit !!!
	}
	return bits[current_bit++];
}
