/*
===============================================================================

	FILE:	mesh.cpp
	
	CONTENTS:
	
    see corresponding header file
	
	PROGRAMMERS:
	
		martin isenburg@cs.unc.edu
	
	COPYRIGHT:
	
		Copyright (C) 2000  Martin Isenburg (isenburg@cs.unc.edu)
		
		This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	
	CHANGE HISTORY:
	
    see corresponding header file
	
===============================================================================
*/
#include "mesh.h"

#include "vector.h"
#include "vertex.h"
#include "normal.h"
#include "spinedge.h"
#include "material.h"
#include "group.h"
#include "vec3fv.h"
#include "positionquantizer.h"

#include <string.h>
#include <math.h>

CFmesh::CFmesh()
{
	translate[0] = 1.0f;
	translate[1] = 1.0f;
	translate[2] = 1.0f;
	scale = 1.0f;

	gate = 0;
	active = 0;
  vs = new CFvector();
  ts = new CFvector();
  hs = new CFvector();
  ps = new CFvector();
  fs = new CFvector();
  es = new CFvector();
  ns = new CFvector();
  ms = new CFvector();
  gs = new CFvector();
  strcpy(name, "no name");
}

CFmesh::CFmesh(char* n)
{
	gate = 0;
	active = 0;
  vs = new CFvector();
  ts = new CFvector();
  hs = new CFvector();
  ps = new CFvector();
  fs = new CFvector();
  es = new CFvector();
  ns = new CFvector();
  ms = new CFvector();
  gs = new CFvector();
  strcpy(name, n);
}

CFmesh::~CFmesh()
{
  int i;
	
  CFvertex* v;
  for (i = 0; i < vs->size(); i++)
	{
		v = (CFvertex*)vs->elementAt(i);
		delete v;
	}
  CFspinEdge* e;
  for (i = 0; i < es->size(); i++)
	{
		e = (CFspinEdge*)es->elementAt(i);
		delete e;
	}
  CFnormal* n;
  for (i = 0; i < ns->size(); i++)
	{
		n = (CFnormal*)ns->elementAt(i);
		delete n;
	}
  CFmaterial* m;
  for (i = 0; i < ms->size(); i++)
	{
		m = (CFmaterial*)ms->elementAt(i);
		delete m;
	}
  CFgroup* g;
  for (i = 0; i < gs->size(); i++)
	{
		g = (CFgroup*)gs->elementAt(i);
		delete g;
	}
  delete vs;
  delete ts;
  delete hs;
  delete ps;
  delete fs;
  delete es;
  delete ns;
  delete ms;
  delete gs;
}

/*
void CFmesh::normalize()
{
  int i;
	
  float Min[3], Max[3], Trans[3], Dim[3], Scale;
  
  VecCopy3fv(Min,((CFvertex*)vs->elementAt(0))->v);
  VecCopy3fv(Max,((CFvertex*)vs->elementAt(0))->v);
	
  for (i = 1; i < vs->size(); i++)
	{
		VecUpdateMinMax3fv(Min,Max,((CFvertex*)vs->elementAt(i))->v);
	}
	
  VecSubtract3fv(Dim,Max,Min);
  if (Dim[0]>=Dim[1] && Dim[0]>=Dim[2]) Scale=2.0f/Dim[0];
  else if (Dim[1]>=Dim[0] && Dim[1]>=Dim[2]) Scale=2.0f/Dim[1];
  else Scale=2.0f/Dim[2];
  VecAdd3fv(Trans,Min,Max);
  VecSelfScalarMult3fv(Trans,0.5f);
  
  for (i = 0; i < vs->size(); i++)
	{
		VecSelfSubtract3fv( ((CFvertex*)vs->elementAt(i))->v, Trans );
		VecSelfScalarMult3fv( ((CFvertex*)vs->elementAt(i))->v, Scale );
	}
}
*/

void CFmesh::normalize()
{
  int i;
	
  float Center[3] = {0.0f, 0.0f, 0.0f};
  for (i = 0; i < vs->size(); i++)
	{
		VecSelfAdd3fv(Center,((CFvertex*)vs->elementAt(i))->v);
	}
	VecSelfScalarDiv3fv(Center, (float)vs->size());

  float SqrtDistance = 0.0f;
  for (i = 0; i < vs->size(); i++)
	{
		SqrtDistance += (float)sqrt(VecDistance3fv(Center,((CFvertex*)vs->elementAt(i))->v));
	}
	SqrtDistance /= vs->size();
  
  for (i = 0; i < vs->size(); i++)
	{
		VecSelfSubtract3fv( ((CFvertex*)vs->elementAt(i))->v, Center );
		VecSelfScalarDiv3fv( ((CFvertex*)vs->elementAt(i))->v, SqrtDistance*SqrtDistance*2.0f );
	}
}

/*
void CFmesh::normalizeUsingActiveVertices()
{
  int i;
	int count_active_vertices;
	
	CFvertex* v;
  float Min[3], Max[3], Trans[3], Dim[3], Scale;
  
	i = 0;
	do
	{
		v = (CFvertex*)vs->elementAt(i);
		i++;
	} while ((i < vs->size()) && (v->active == 0));

	if (i == vs->size()) return;

  VecCopy3fv(Min,v->v);
  VecCopy3fv(Max,v->v);

	count_active_vertices = 1;
	do
	{
		v = (CFvertex*)vs->elementAt(i);
		if (v->active)
		{
			VecUpdateMinMax3fv(Min,Max,((CFvertex*)vs->elementAt(i))->v);
			count_active_vertices++;
		}
		i++;
	} while (i < vs->size());

	if (count_active_vertices < 4) return;

  VecSubtract3fv(Dim,Max,Min);
  if (Dim[0]>=Dim[1] && Dim[0]>=Dim[2]) Scale=2.0f/Dim[0];
  else if (Dim[1]>=Dim[0] && Dim[1]>=Dim[2]) Scale=2.0f/Dim[1];
  else Scale=2.0f/Dim[2];
  VecAdd3fv(Trans,Min,Max);
  VecSelfScalarMult3fv(Trans,0.5f);
  
  for (i = 0; i < vs->size(); i++)
	{
		VecSelfSubtract3fv( ((CFvertex*)vs->elementAt(i))->v, Trans );
		VecSelfScalarMult3fv( ((CFvertex*)vs->elementAt(i))->v, Scale );
	}
}
*/

void CFmesh::normalizeUsingActiveVertices()
{
  int i;
	
	int num_active = 0;
  float Center[3] = {0.0f, 0.0f, 0.0f};
  for (i = 0; i < vs->size(); i++)
	{
		if (((CFvertex*)vs->elementAt(i))->active)
		{
			VecSelfAdd3fv(Center,((CFvertex*)vs->elementAt(i))->v);
			num_active++;
		}
	}
	if (num_active < 4) return;

	VecSelfScalarDiv3fv(Center, (float)num_active);

  float SqrtDistance = 0.0f;
  for (i = 0; i < vs->size(); i++)
	{
		if (((CFvertex*)vs->elementAt(i))->active)
		{
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,((CFvertex*)vs->elementAt(i))->v));
		}
	}
	SqrtDistance /= num_active;
  
  for (i = 0; i < vs->size(); i++)
	{
		VecSelfSubtract3fv( ((CFvertex*)vs->elementAt(i))->v, Center );
		VecSelfScalarDiv3fv( ((CFvertex*)vs->elementAt(i))->v, SqrtDistance*SqrtDistance*2.0f );
	}
}

void CFmesh::normalizeUsingCell(CFspinEdge* cell)
{
  int i;
	
	if (cell && cell->group != -1)
	{
	  float Center[3] = {0.0f, 0.0f, 0.0f};
	  float SqrtDistance = 0.0f;
		if (hs->size() > 0) // it is a hexahedron
		{
			CFspinEdge *e0 = cell;
			CFspinEdge *e1 = e0->next->next->next;
			CFspinEdge *e2 = e0->next->next;
			CFspinEdge *e3 = e0->next;
			CFspinEdge *e4 = e2->spin->inv->next->next->spin->inv;
			CFspinEdge *e5 = e4->next->next->next;
			CFspinEdge *e6 = e4->next->next;
			CFspinEdge *e7 = e4->next;
			VecSelfAdd3fv(Center,e0->vertex->v);
			VecSelfAdd3fv(Center,e1->vertex->v);
			VecSelfAdd3fv(Center,e2->vertex->v);
			VecSelfAdd3fv(Center,e3->vertex->v);
			VecSelfAdd3fv(Center,e4->vertex->v);
			VecSelfAdd3fv(Center,e5->vertex->v);
			VecSelfAdd3fv(Center,e6->vertex->v);
			VecSelfAdd3fv(Center,e7->vertex->v);
			VecSelfScalarDiv3fv(Center, 8);
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,e0->vertex->v));
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,e1->vertex->v));
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,e2->vertex->v));
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,e3->vertex->v));
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,e4->vertex->v));
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,e5->vertex->v));
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,e6->vertex->v));
			SqrtDistance += (float)sqrt(VecDistance3fv(Center,e7->vertex->v));
			SqrtDistance /= 8;
		}
		else
		{
		}  
		for (i = 0; i < vs->size(); i++)
		{
			VecSelfSubtract3fv( ((CFvertex*)vs->elementAt(i))->v, Center );
			VecSelfScalarDiv3fv( ((CFvertex*)vs->elementAt(i))->v, SqrtDistance*SqrtDistance*2.0f );
		}
	}
}

void CFmesh::computePerFaceNormals()
{
  int i;
	float tmpN[3];
  CFnormal* n;
  CFspinEdge* e;
	
  for (i = 0; i < es->size(); i++)
  {
    e = (CFspinEdge*) es->elementAt(i);
    e->normal = 0;
  }

  for (i = 0; i < ns->size(); i++)
  {
    n = (CFnormal*) ns->elementAt(i);
    delete n;
  }
  ns->removeAllElements();
  
	ns->ensureCapacity(fs->size());
	
  for (i = 0; i < fs->size(); i++)
  {
    e = (CFspinEdge*) fs->elementAt(i);
		n = new CFnormal(0.0, 0.0, 0.0);
		ns->addElement((void*) n);

		e->normal = n;
		e->next->normal = n;
		e->next->next->normal = n;
		e->next->next->next->normal = n;

		VecCwNormal3fv(tmpN, e->vertex->v, e->next->vertex->v, e->next->next->vertex->v);
		VecSelfAdd3fv(n->n, tmpN);
		VecCwNormal3fv(tmpN, e->next->vertex->v, e->next->next->vertex->v, e->next->next->next->vertex->v);
		VecSelfAdd3fv(n->n, tmpN);
  }
	
  for (i = 0; i < ns->size(); i++)
  {
    n = (CFnormal*) ns->elementAt(i);    
    VecSelfNormalize3fv(n->n);
  }
}

void CFmesh::enQuantize(PositionQuantizer* pq)
{
  int i;
  CFvertex* v;

  for (i = 0; i < vs->size(); i++)
  {
		v = (CFvertex*)vs->elementAt(i);
		pq->UpdateMinMax(v->v);
  }

	pq->SetupQuantizer();

//	cout << "m_fDeQuantizeMultiplier: " << pq->m_fDeQuantizeMultiplier << endl;

  for (i = 0; i < vs->size(); i++)
  {
		v = (CFvertex*)vs->elementAt(i);
		pq->EnQuantize(v->v, v->vq);
  }
}

void CFmesh::deQuantize(PositionQuantizer* pq)
{
  int i;
  CFvertex* v;

	pq->SetupQuantizer();

  for (i = 0; i < vs->size(); i++)
  {
		v = (CFvertex*)vs->elementAt(i);
		pq->DeQuantize(v->vq, v->v);
  }
}
