/*
===============================================================================

	FILE:	mesh.h
	
	CONTENTS:
	
		Cell Fixer
	
	PROGRAMMERS:
	
		martin isenburg@cs.unc.edu
	
	COPYRIGHT:
	
		(c) 2002 martin isenburg@cs.unc.edu
		
		This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	
	CHANGE HISTORY:
	
		20 May 2000 -- created
	
===============================================================================
*/
#ifndef CF_MESH_H
#define CF_MESH_H

class CFvector;
class CFspinEdge;
class CFmaterial;
class PositionQuantizer;

class CFmesh
{
public:
  char name[128];
	
	float translate[3];
	float scale;

  CFvector *vs; // vertices
  CFvector *ts; // tetrahedrons
  CFvector *hs; // hexahedrons
  CFvector *ps; // polyhedrons
  CFvector *fs; // faces
  CFvector *es; // edges
  CFvector *ns; // normals
  CFvector *ms; // materials
  CFvector *gs; // groups
  
  int NumTetrahedra;
  int NumHoles;
  int NumHandles;
  int NumComponents;

	int interior_edges;
	int total_edges;

	int interior_faces;
	int total_faces;
  
	int interior_vertices;
	int total_vertices;

	CFmaterial* boundary_material;
	CFmaterial* interior_material;

  CFspinEdge* gate;
  CFspinEdge* active;
  
  CFmesh();
  CFmesh(char* n);
  ~CFmesh();
	
  void normalize();
	void normalizeUsingActiveVertices();
	void normalizeUsingCell(CFspinEdge* cell);
	
  void computePerFaceNormals();

	void enQuantize(PositionQuantizer* pq);
	void deQuantize(PositionQuantizer* pq);
};

#endif
