/*
===============================================================================

	FILE:	vec3fv.cpp
	
	CONTENTS:
	
    see corresponding header file
	
	PROGRAMMERS:
	
		Martin Isenburg@cs.unc.edu  (with ideas stolen from Kenny Hoff)
	
	COPYRIGHT:
	
		Copyright (C) 2000  Martin Isenburg (isenburg@cs.unc.edu)
		
		This software is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	
	CHANGE HISTORY:
	
    see corresponding header file
	
===============================================================================
*/
#include "vec3fv.h"
#include <math.h>

void VecZero3fv(float v[3])
{
  v[0] = 0.0f;
  v[1] = 0.0f;
  v[2] = 0.0f;
}

void VecSet3fv(float v[3], float x, float y, float z)
{
  v[0] = x;
  v[1] = y;
  v[2] = z;
}

void VecCopy3fv(float v[3], float a[3])
{
  v[0] = a[0];
  v[1] = a[1];
  v[2] = a[2];
}

void VecSelfScalarMult3fv(float v[3], float s)
{
  v[0] *= s;
  v[1] *= s;
  v[2] *= s;
}

void VecSelfScalarDiv3fv(float v[3], float s)
{
  v[0] /= s;
  v[1] /= s;
  v[2] /= s;
}

void VecSelfNegate3fv(float v[3])
{
  v[0] = -v[0];
  v[1] = -v[1];
  v[2] = -v[2];
}

void VecScalarMult3fv(float v[3], float a[3], float s)
{
  v[0] = a[0]*s;
  v[1] = a[1]*s;
  v[2] = a[2]*s;
}

void VecScalarDiv3fv(float v[3], float a[3], float s)
{
  v[0] = a[0]/s;
  v[1] = a[1]/s;
  v[2] = a[2]/s;
}

void VecNegate3fv(float v[3], float a[3])
{
  v[0] = -a[0];
  v[1] = -a[1];
  v[2] = -a[2];
}

void VecSelfAdd3fv(float v[3], float a[3])
{
  v[0] += a[0];
  v[1] += a[1];
  v[2] += a[2];
}

void VecSelfSubtract3fv(float v[3], float a[3])
{
  v[0] -= a[0];
  v[1] -= a[1];
  v[2] -= a[2];
}

void VecAdd3fv(float v[3], float a[3], float b[3])
{
  v[0] = a[0] + b[0];
  v[1] = a[1] + b[1];
  v[2] = a[2] + b[2];
}

void VecAdd3fv(float v[3], float a[3], float b[3], float c[3])
{
  v[0] = a[0] + b[0] + c[0];
  v[1] = a[1] + b[1] + c[1];
  v[2] = a[2] + b[2] + c[2];
}

void VecSubtract3fv(float v[3], float a[3], float b[3])
{
  v[0] = a[0] - b[0];
  v[1] = a[1] - b[1];
  v[2] = a[2] - b[2];
}

void VecSelfAddScalarMult3fv(float v[3], float a[3], float s)
{
  v[0] += s*a[0];
  v[1] += s*a[1];
  v[2] += s*a[2];
}

void VecAddScalarMult3fv(float v[3], float a[3], float b[3], float s)
{
  v[0] = a[0]+s*b[0];
  v[1] = a[1]+s*b[1];
  v[2] = a[2]+s*b[2];
}

float VecLength3fv(float v[3])
{
  return( (float)sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]) );
}

float VecDistance3fv(float a[3], float b[3])
{
  return( (float)sqrt((a[0]-b[0])*(a[0]-b[0]) + (a[1]-b[1])*(a[1]-b[1]) + (a[2]-b[2])*(a[2]-b[2])) );
}

float VecSquaredLength3fv(float v[3])
{
  return( v[0]*v[0] + v[1]*v[1] + v[2]*v[2] );
}

float VecSquaredDistance3fv(float a[3], float b[3])
{
  return( (a[0]-b[0])*(a[0]-b[0]) + (a[1]-b[1])*(a[1]-b[1]) + (a[2]-b[2])*(a[2]-b[2]) );
}

void VecNormalize3fv(float v[3], float a[3])
{
  float l = VecLength3fv(a);
  v[0] = a[0]/l;
  v[1] = a[1]/l;
  v[2] = a[2]/l;
}

void VecSelfNormalize3fv(float v[3])
{
  float l = VecLength3fv(v);
  v[0] /= l;
  v[1] /= l;
  v[2] /= l;
}

float VecDotProd3fv(float a[3], float b[3])
{
  return( a[0]*b[0] + a[1]*b[1] + a[2]*b[2] );
}

void VecCrossProd3fv(float v[3], float a[3], float b[3]) // c = a X b
{ 
  VecSet3fv(v, a[1]*b[2]-a[2]*b[1], a[2]*b[0]-a[0]*b[2], a[0]*b[1]-a[1]*b[0]);
}

void VecUpdateMinMax3fv(float min[3], float max[3], float v[3])
{
  if (v[0]<min[0]) min[0]=v[0]; else if (v[0]>max[0]) max[0]=v[0];
  if (v[1]<min[1]) min[1]=v[1]; else if (v[1]>max[1]) max[1]=v[1];
  if (v[2]<min[2]) min[2]=v[2]; else if (v[2]>max[2]) max[2]=v[2];
}

int VecEqual3fv(float a[3], float b[3])
{
  return( a[0]==b[0] && a[1]==b[1] && a[2]==b[2] );
}

void VecCcwNormal3fv(float n[3], float a[3], float b[3], float c[3])
{
  float ab[3], ac[3];
  VecSubtract3fv(ab,b,a);
  VecSubtract3fv(ac,c,a);
  VecCrossProd3fv(n,ab,ac);
}

void VecCwNormal3fv(float n[3], float a[3], float b[3], float c[3])
{
  float ab[3], ac[3];
  VecSubtract3fv(ab,b,a);
  VecSubtract3fv(ac,c,a);
  VecCrossProd3fv(n,ac,ab);
}

float VecTriangleArea3fv(float a[3], float b[3], float c[3])
{
	float n[3];
	VecCcwNormal3fv(n, a, b, c);
	return float(0.5*VecLength3fv(n));
}

void VecCcwNormNormal3fv(float n[3], float a[3], float b[3], float c[3])
{
  float ab[3], ac[3];
  VecSubtract3fv(ab,b,a);
  VecSubtract3fv(ac,c,a);
  VecCrossProd3fv(n,ab,ac);
  VecSelfNormalize3fv(n);
}

void VecCwNormNormal3fv(float n[3], float a[3], float b[3], float c[3])
{
  float ab[3], ac[3];
  VecSubtract3fv(ab,b,a);
  VecSubtract3fv(ac,c,a);
  VecCrossProd3fv(n,ac,ab);
  VecSelfNormalize3fv(n);
}
